from typing import Dict, Type

from pydantic import BaseModel


def _create_dict(schema_head: Dict, definitions: Dict):
    schema_dict = {}
    if 'type' in schema_head and schema_head['type'] == 'object':
        for key, value in schema_head['properties'].items():
            if '$ref' in value:
                sub_schema = value['$ref'].replace('#/$defs/', '')
                schema_dict[key] = _create_dict(definitions[sub_schema], definitions)
            else:
                schema_dict[key] = None
    else:
        return None
    return schema_dict


def get_schema(config_cls: Type[BaseModel]):
    schema = config_cls.model_json_schema()
    definitions = schema['$defs'] if '$defs' in schema else {}
    return _create_dict(schema, definitions)



# module of all supported annotations
import os
from typing import Any

import pydantic
from pydantic import Field


def EnvField(default: Any, *, env_var: str, **kwargs):
    return Field(default, default_factory=lambda: os.environ.get(env_var, default=None), **kwargs)


def Secret(env_var: str):
    return EnvField(..., env_var=env_var)


def OpenRange(gt: float = None, lt: float = None) -> Field:
    return Field(..., gt=gt, lt=lt)


def ClosedRange(ge: float = None, le: float = None):
    return Field(..., ge=ge, le=le)


def MixedRange(gt: float = None, ge: float = None, lt: float = None, le: float = None):
    return Field(..., gt=gt, ge=ge, lt=lt, le=le)


__all__ = list(pydantic.__all__) + [Secret, OpenRange, ClosedRange, MixedRange]
